unit ColorCode;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Menus, ComCtrls, OleCtrls, SPIRITLib_TLB, ToolWin, Buttons,
  ExtCtrls;

procedure ShowColorCode(TheEditor: TRichEdit; firstline, numboflines:integer);

procedure HideColorCode(TheEditor: TRichEdit);

implementation

uses MainUnit, Editor, Preferences;

var Ed:TRichEdit;




{Character Code types}

var newchartype:array[1..1023] of integer;  // new type of each char

procedure SetCharTypes(l:integer);
{Sets the characters to the correct type}
var start,i,cpos,len:integer;
begin
  start := Ed.Perform(EM_LINEINDEX,l,0);
  len := Length(Ed.Lines[l]);
  i:=1;
  while i <= len do
  begin
    cpos:=i;
    repeat i:=i+1 until  (i > len) or (newchartype[i] <> newchartype[cpos]);
    Ed.SelStart:=start+cpos-1;
    Ed.SelLength := i-cpos;
    if (not (cc_style[newchartype[cpos]] = Ed.SelAttributes.Style)) or
       (not (cc_color[newchartype[cpos]] = Ed.SelAttributes.Color)) or
       (not (caItalic in Ed.SelAttributes.ConsistentAttributes)) or
       (not (caBold in Ed.SelAttributes.ConsistentAttributes)) or
       (not (caColor in Ed.SelAttributes.ConsistentAttributes)) then
    begin
      Ed.SelAttributes.Style := cc_style[newchartype[cpos]];
      Ed.SelAttributes.Color := cc_color[newchartype[cpos]];
    end;
  end;
end;

procedure ColorIt(start,numb,code:integer);
{Colors numb chars starting at start in code color}
var i:integer;
begin
  for i:=start to start+numb-1 do newchartype[i] := code;
end;

function IsKeyChar(ch:char):boolean;
{Returns whether ch is a character in a keyword name}
begin
  IsKeyChar :=
      ((ch>= 'a') and (ch<= 'z')) or
      ((ch>= 'A') and (ch<= 'Z')) or
      ((ch>= '0') and (ch<= '9')) or
      (ch = '_');
end;

function IsKeyWord(str:string):boolean;
{Checks whether str is a keyword}
var i:integer;
begin
  IsKeyWord:=false;
  for i:=1 to cc_keynumb do
    if CompareStr(str,cc_keywords[i]) = 0 then
      begin IsKeyWord:=true; Exit; end;
end;

function IsCommand(str:string):boolean;
{Checks whether str is a command}
var i:integer;
begin
  IsCommand:=false;
  for i:=1 to cc_commandnumb do
    if CompareStr(str,cc_commands[i]) = 0 then
      begin IsCommand:=true; Exit; end;
end;

function IsConstant(str:string):boolean;
{Checks whether str is a constant}
var i:integer;
begin
  IsConstant:=false;
  for i:=1 to cc_constantnumb do
    if CompareStr(str,cc_constants[i]) = 0 then
      begin IsConstant:=true; Exit; end;
end;

function IsCommentLineEnd(l:integer):boolean;
{Returns whether line l ends with a running comment}
var len:integer;
begin
  if l<0 then
    begin IsCommentLineEnd := false; Exit; end;
  len := Length(Ed.Lines[l]);
  if len = 0 then
    begin IsCommentLineEnd := IsCommentLineEnd(l-1); Exit; end;
  if (len>=2) and (Ed.Lines[l][len] = '/') and (Ed.Lines[l][len-1] = '*') then
    begin IsCommentLineEnd := false; Exit; end;
  Ed.SelStart:=Ed.Perform(EM_LINEINDEX,l,0)+len-1;
  Ed.SelLength := 1;
  IsCommentLineEnd := (Ed.SelAttributes.Color = cc_color[CC_COMMENTL]);
end;

function IsCommentLineStart(l:integer):boolean;
{Returns whether line l starts with a running comment}
var len:integer;
begin
  if l>= Ed.Lines.Count then
    begin IsCommentLineStart := false; Exit; end;
  len := Length(Ed.Lines[l]);
  if len = 0 then
    begin IsCommentLineStart := IsCommentLineStart(l+1); Exit; end;
  if (len>=2) and (Ed.Lines[l][1] = '/') and (Ed.Lines[l][2] = '*') then
    begin IsCommentLineStart := false; Exit; end;
  Ed.SelStart:=Ed.Perform(EM_LINEINDEX,l,0);
  Ed.SelLength := 1;
  IsCommentLineStart := (Ed.SelAttributes.Color = cc_color[CC_COMMENTL]);
end;

procedure ColorCodeLine(l:integer);
{Color codes line l}
var
  len:integer;          {Length of the line}
  curpos:integer;       {Current position in the line}
  str:string;           {Copy of the current line}
  str2:string;          {Temporary string}
  ttt:integer;          {Temporary index}
begin
  if (l<0) or (l>=Ed.Lines.Count) then Exit;
  str:=Ed.Lines[l];
  len := Length(str);
  ColorIt(1,len,CC_NORMAL);

  {Handle the defines}
  curpos:=1;
  while curpos <= len do
  begin
    if str[curpos] = '#' then
    begin
      ColorIt(curpos,len-curpos+1,CC_DEFINE);
      curpos:=len+1;
    end else
      curpos:=curpos+1;
  end;

  {Handle the comments}
  curpos:=1;
    {Check whether comment was running on from the previous line}
  if IsCommentLineEnd(l-1) then
  begin
    curpos := 3;
    while curpos <= len do
      if (str[curpos-1] = '*') and (str[curpos] = '/') then
        break
      else
        curpos:=curpos+1;
    if curpos > len then curpos:=curpos-1;
    ColorIt(1, curpos,CC_COMMENTL);
    curpos:=curpos+1;
  end;
    {Handle the rest of the line (if any)}
  while curpos <= len do
  begin
    {Check for a single line comment}
    if (curpos<len) and (str[curpos] = '/') and (str[curpos+1] = '/') then
    begin
      ColorIt(curpos,len-curpos+1,CC_COMMENTS);
      curpos:=len+1;
    end;
    {Check for a normal comment}
    if (curpos<len) and (str[curpos] = '/') and (str[curpos+1] = '*') then
    begin
      ttt := curpos;
      curpos := curpos+3;
      while curpos <= len do
        if (str[curpos-1] = '*') and (str[curpos] = '/') then
          break
        else
          curpos:=curpos+1;
      if curpos > len then curpos:=curpos-1;
      ColorIt(ttt, curpos-ttt+1,CC_COMMENTL);
    end;
    curpos:=curpos+1;
  end;

  {Handle the fields}
  curpos:=1;
  while curpos <= len do
  begin
    {Check for a single line comment}
    if (curpos<len) and (str[curpos] = '"')then
    begin
      ttt := curpos;
      curpos := curpos+1;
      while curpos <= len do
        if (str[curpos] = '"') then
          break
        else
          curpos:=curpos+1;
      if curpos > len then curpos:=curpos-1;
      if (newchartype[ttt] = CC_NORMAL) then
        ColorIt(ttt, curpos-ttt+1,CC_FIELD);
    end;
    curpos:=curpos+1;
  end;

  {Check for keywords}
  curpos:=1;
  while curpos <= len do
  begin
    if IsKeyChar(str[curpos]) then
    begin
      ttt:=curpos;
      str2:='';
      while (curpos <= len) and IsKeyChar(str[curpos]) do
        begin  str2:=str2+str[curpos]; curpos:=curpos+1; end;
      if IsKeyWord(str2) then
      begin
        if (newchartype[ttt] = CC_NORMAL) then
          ColorIt(ttt,curpos-ttt,CC_KEYWORD)
        else if (newchartype[ttt] = CC_DEFINE) then
          ColorIt(ttt,curpos-ttt,CC_DEFINEKEY);
      end else if IsCommand(str2) then
      begin
        if (newchartype[ttt] = CC_NORMAL) then
          ColorIt(ttt,curpos-ttt,CC_COMMAND)
        else if (newchartype[ttt] = CC_DEFINE) then
          ColorIt(ttt,curpos-ttt,CC_DEFINECOM);
      end else if IsConstant(str2) then
      begin
        if (newchartype[ttt] = CC_NORMAL) then
          ColorIt(ttt,curpos-ttt,CC_CONSTANT)
        else if (newchartype[ttt] = CC_DEFINE) then
          ColorIt(ttt,curpos-ttt,CC_DEFINECONST);
      end;
    end else begin
      curpos:=curpos+1;
    end;
  end;

  {Draw the line in the correct colors}
  SetCharTypes(l);
end;

procedure ShowColorCode(TheEditor: TRichEdit; firstline, numboflines:integer);
{Color codes numbof lines lines, starting at firstline
 in RichEdit TheEditor}
var i,ss,sl,topline:integer;
    TempControl:TWinControl;
begin
  {Start it}
  Dont_Update := true;
  Ed := TheEditor;
  Ed.HideSelection:=true;
  TempControl :=TEditorForm(Ed.Parent).ActiveControl;
  TEditorForm(Ed.Parent).ActiveControl := nil;
  if numboflines>1 then   // TO CORRECT SOME WEIRD BUG
  begin
    topline := Ed.Perform(EM_GETFIRSTVISIBLELINE,0,0);
    Ed.Perform(EM_LINESCROLL,0,firstline+numboflines-1-topline);
  end;
  ss := Ed.SelStart;
  sl := Ed.SelLength;
  {Code the lines}
  for i:= firstline to firstline + numboflines - 1 do
    ColorCodeLine(i);
  i:= firstline+numboflines;
  {Handle running comments on further lines}
  while (i<Ed.Lines.Count) and
        (IsCommentLineEnd(i-1) <> IsCommentLineStart(i)) do
  begin
    ColorCodeLine(i);
    i:=i+1;
  end;
  {Clean up}
  Ed.SelStart := ss;
  Ed.SelLength := sl;
  if numboflines>1 then   // TO CORRECT SOME WEIRD BUG
  begin
    Ed.Perform(EM_LINESCROLL,0,topline-Ed.Perform(EM_GETFIRSTVISIBLELINE,0,0));
  end;
  Ed.HideSelection:=false;
  TEditorForm(Ed.Parent).ActiveControl := TempControl;
  Dont_Update:=false;
end;

procedure HideColorCode(TheEditor: TRichEdit);
var ss,sl:integer;
    TempControl:TWinControl;
begin
  {Start it}
  Dont_Update := true;
  TheEditor.HideSelection:=true;
  TempControl :=TEditorForm(Ed.Parent).ActiveControl;
  TEditorForm(Ed.Parent).ActiveControl := nil;
  ss := TheEditor.SelStart;
  sl := TheEditor.SelLength;
  {Do it}
  TheEditor.SelectAll;
  TheEditor.SelAttributes.Style := cc_style[CC_NORMAL];
  TheEditor.SelAttributes.Color := cc_color[CC_NORMAL];
  {Clean up}
  TheEditor.SelStart := ss;
  TheEditor.SelLength := sl;
  TheEditor.HideSelection:=false;
  TEditorForm(Ed.Parent).ActiveControl := TempControl;
  Dont_Update:=false;
end;

end.
