unit Editor;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, Menus;

type
  TEditorForm = class(TForm)
    TheEditor: TRichEdit;
    TheErrors: TListBox;
    procedure TheErrorsClick(Sender: TObject);
    procedure TheEditorChange(Sender: TObject);
    procedure TheEditorSelectionChange(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormShow(Sender: TObject);
    procedure TheEditorKeyPress(Sender: TObject; var Key: Char);
    procedure TheEditorKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    { Private declarations }
  public
    {File handling}
    FileName:string;                   // Name of File in editor
    IsNew:boolean;                     // Whether it is a new file
    procedure NewFile(fname:string);   // Create a new file in the editor
    procedure OpenFile(fname:string);  // Open an existing file
    procedure SaveFile;                // Saves the file
    procedure SaveFileAs(fname:string);// Saves the file as fname
    procedure InsertFile(fname:string);// Insert file at cursor
    procedure PrintFile;               // Prints the file

    {Editing}
    function  CanUndo:boolean;         // Whether you can undo something
    function  Selected:boolean;        // Whether something is selected
    procedure Undo;
    procedure CutSel;
    procedure CopySel;
    procedure Paste;
    procedure DeleteSel;
    procedure SelectAll;
    procedure GotoLine;
    function FindNext(curpos:integer; findstr:string;
                  wholeword, casesens: boolean):integer;
    function Findit(findstr,repstr:string;
                  wholeword, casesens, repl, all:boolean):boolean;
    procedure NextField;
    procedure RefreshScreen;
    procedure AddConstructString(constr:string);
    procedure AddConstruct(numb:integer);
  end;

var
  EditorForm: TEditorForm;

implementation

{$R *.DFM}

uses MainUnit, Preferences, ColorCode, GotoLine, ConstructUnit;


{File Handling routines}

procedure TEditorForm.NewFile(fname:string);
begin
  FileName:=fname;
  IsNew:=true;
  Caption:=ExtractFileName(fname);
  TheEditor.Modified:=false;
  MainForm.SaveBtn.Enabled := false;
  TheEditorSelectionChange(TheEditor);
end;

procedure TEditorForm.OpenFile(fname:string);
begin
  FileName:=fname;
  IsNew:=false;
  Caption:=ExtractFileName(fname);
  if FileExists(fname) then
  begin
    Dont_Update := true;     {Seems neccesary???}
    TheEditor.Lines.LoadFromFile(fname);
    Dont_Update := false;    {Seems neccesary???}
    if ColorCoding then ShowColorCode(TheEditor,0,TheEditor.Lines.Count);
  end;
  TheEditor.Modified:=false;
  MainForm.SaveBtn.Enabled := false;
  TheEditorSelectionChange(TheEditor);
end;

procedure TEditorForm.SaveFile;
begin
  SaveFileAs(FileName);
end;

procedure TEditorForm.SaveFileAs(fname:string);
var backfname:string;
begin
  FileName:=fname;
  IsNew:=false;
  Caption:=ExtractFileName(fname);
  if SaveBackup and FileExists(fname) then
  begin
    backfname := ChangeFileExt(fname,'.bak');
    DeleteFile(backfname);
    RenameFile(fname,backfname);
  end;
  TheEditor.Lines.SaveToFile(fname);
  TheEditor.Modified:=false;
  MainForm.SaveBtn.Enabled := false;
end;

procedure TEditorForm.InsertFile(fname:string);
var l1,l2:integer;
begin
  MainForm.TempEditor.Lines.LoadFromFile(fname);
  l1:= TheEditor.Perform(EM_LINEFROMCHAR,TheEditor.SelStart,0);
  TheEditor.SetSelTextBuf(MainForm.TempEditor.Lines.GetText);
  l2:= TheEditor.Perform(EM_LINEFROMCHAR,TheEditor.SelStart,0);
  if ColorCoding then ShowColorCode(TheEditor,l1,l2-l1+1);
end;

procedure TEditorForm.PrintFile;
begin
  TheEditor.Print(Caption);
end;

{Edit routines}

function TEditorForm.CanUndo:boolean;
begin
  Result := TheEditor.Perform(EM_CANUNDO,0,0) <> 0;
end;

function TEditorForm.Selected:boolean;
begin
  Result := TheEditor.SelLength >0;
end;

procedure TEditorForm.Undo;
begin
  TheEditor.Perform(EM_UNDO,0,0);
end;

procedure TEditorForm.CutSel;
var l1:integer;
begin
  TheEditor.CutToClipboard();
  l1:= TheEditor.Perform(EM_LINEFROMCHAR,TheEditor.SelStart,0);
  if ColorCoding then ShowColorCode(TheEditor,l1,1);
end;

procedure TEditorForm.CopySel;
begin
  TheEditor.CopyToClipboard();
end;

procedure TEditorForm.Paste;
var l1,l2:integer;
begin
  l1:= TheEditor.Perform(EM_LINEFROMCHAR,TheEditor.SelStart,0);
  TheEditor.PasteFromClipboard();
  l2:= TheEditor.Perform(EM_LINEFROMCHAR,TheEditor.SelStart,0);
  if ColorCoding then ShowColorCode(TheEditor,l1,l2-l1+1);
end;

procedure TEditorForm.DeleteSel;
var l1:integer;
begin
  TheEditor.ClearSelection();
  l1:= TheEditor.Perform(EM_LINEFROMCHAR,TheEditor.SelStart,0);
  if ColorCoding then ShowColorCode(TheEditor,l1,1);
end;

procedure TEditorForm.SelectAll;
begin
  TheEditor.SelectAll();
end;

procedure TEditorForm.GotoLine;
begin
  if GotoForm.ShowModal() = mrOK then
  begin
    TheEditor.SetFocus;
    TheEditor.SelStart := TheEditor.Perform(EM_LINEINDEX,theline-1,0);
    TheEditor.SelLength := 0;
  end;
end;

function TEditorForm.FindNext(curpos:integer; findstr:string;
                  wholeword, casesens: boolean):integer;
{ Returns the first position, starting at curpos where to find it. }
var foundat, endpos: integer;
    st: TSearchTypes;
begin
  with TheEditor do
  begin
    endpos := Length(Text) - curpos;
    st := [];
    if wholeword then st := st + [stWholeWord];
    if casesens then st := st + [stMatchCase];
    foundat := FindText(findstr, curpos, endpos, st);
    if foundat <> -1 then
    begin
      SetFocus;
      SelStart := foundat;
      SelLength := Length(findstr);
    end;
  end;
  FindNext := foundat;
end;

function TEditorForm.Findit(findstr,repstr:string;
                  wholeword, casesens, repl, all:boolean):boolean;
{Finds all occurences and replaces them if required}
var start:integer;
    foundat:integer;
begin
  {correct starting position}
  if repl then
    start := TheEditor.SelStart
  else
    start := TheEditor.SelStart+TheEditor.SelLength;
  {Go for it}
  while true do
  begin
    {Find next}
    foundat := FindNext(start,findstr,wholeword,casesens);
    if foundat >= 0 then
    begin
      {change it if required}
      if repl then TheEditor.SelText:=repstr;
      if (not repl) or (not all) then
      begin
        Findit := true; Exit;
      end else begin
        start:= start+TheEditor.SelLength;
      end;
    end else begin
      {check whether we should continue from the start}
      start:=0;
      if MessageDlg('Pattern not found; continue from start?',
            mtConfirmation, [mbYes, mbNo], 0) <> mrYes then
        begin  TheEditor.SelLength:=0; FindIt:=false; Exit; end;
    end;
  end;
end;

procedure TEditorForm.NextField;
var start,found1,found2: integer;
begin
  with TheEditor do
  begin
    start := SelStart+SelLength;
    found1 := FindText('"',start,Length(Lines.Text)-start,[]);
    if found1 <> -1 then
    begin
      found2 := FindText('"',found1+1,Length(Lines.Text)-found1,[]);
      if found2 <> -1 then
      begin
        SetFocus;
        SelStart := found1;
        SelLength := found2-found1+1;
      end;
    end;
  end;
end;

procedure TEditorForm.RefreshScreen;
begin
  if ColorCoding then ShowColorCode(TheEditor,0,TheEditor.Lines.Count);
end;

procedure TEditorForm.AddConstructString(constr:string);
var str:string;
    i,j,tt,lnumb,l1,l2,curpos:integer;
    escaped,fieldexists:boolean;
begin
  lnumb:= TheEditor.Perform(EM_LINEFROMCHAR,TheEditor.SelStart,0);
  tt := TheEditor.SelStart - TheEditor.Perform(EM_LINEINDEX,lnumb,0);
  fieldexists:=false;
  escaped:=false;
  str:='';
  for i:=1 to Length(constr) do
  begin
    if escaped then
    begin
      if constr[i] = '\' then str:=str+'\';
      if constr[i] = '<' then tt:=tt-2;
      if constr[i] = '>' then tt:=tt+2;
      if constr[i] in ['=','<','>'] then
      begin
        str:=str+#13+#10;
        for j:=1 to tt do str:=str+' ';
      end;
      escaped := false;
    end else begin
      if constr[i] = '"' then fieldexists := true;
      if constr[i] = '\' then
        escaped := true
      else
        str:=str+constr[i];
    end;
  end;
  MainForm.SetFocus;
  TheEditor.SetFocus;
  curpos := TheEditor.SelStart;
  l1:= TheEditor.Perform(EM_LINEFROMCHAR,TheEditor.SelStart,0);
  TheEditor.SetSelTextBuf(PChar(str));
  l2:= TheEditor.Perform(EM_LINEFROMCHAR,TheEditor.SelStart,0);
  if ColorCoding then ShowColorCode(TheEditor,l1,l2-l1+1);
  if fieldexists then
  begin
    TheEditor.SelStart := curpos;
    NextField;
  end;
end;

procedure TEditorForm.AddConstruct(numb:integer);
begin
  AddConstructString(constructions[numb]);
end;


{Event Handlers}

procedure TEditorForm.TheErrorsClick(Sender: TObject);
var i,lpos,epos,lnumb,c:integer;
    str:string;
begin
  lnumb := 0;
  for i := TheErrors.ItemIndex downto 0 do
  begin
    epos := Pos('line ',TheErrors.Items[i]);
    if epos >0 then
    begin
     str := Copy(TheErrors.Items[i],epos+4,5);
     Val(str,lnumb,c);
     break;
    end;
  end;
  if lnumb >0 then
  begin
    lpos:=0;
    for i:= 0 to lnumb-2 do lpos:=lpos + Length(TheEditor.Lines[i]) + 2;
    TheEditor.SelStart := lpos;
    TheEditor.SelLength := Length(TheEditor.Lines[lnumb-1]);
  end;
  TheEditor.SetFocus;
end;

procedure TEditorForm.TheEditorChange(Sender: TObject);
begin
  {Avoid changes while changing}
  if Dont_Update then Exit;

  {Set the Save button}
  MainForm.SaveBtn.Enabled := TheEditor.Modified;

  {ColorCode the line}
  if ColorCoding then ShowColorCode(TheEditor,
          TheEditor.Perform(EM_LINEFROMCHAR,TheEditor.SelStart,0),1);
end;

procedure TEditorForm.TheEditorSelectionChange(Sender: TObject);
var lpos,lnumb,ltot,len:integer;
begin
  {Avoid work while changing}
  if Dont_Update then Exit;

  {Fill information on current position in the status bar}
  lnumb:= TheEditor.Perform(EM_LINEFROMCHAR,TheEditor.SelStart,0);
  lpos := TheEditor.SelStart - TheEditor.Perform(EM_LINEINDEX,lnumb,0);
  ltot := TheEditor.Perform(EM_GETLINECOUNT,0,0);
  MainForm.StatusBar1.Panels[0].Text :=
     Format('%d/%d: %3d',[lnumb+1,ltot,lpos+1]);

  {Change selection of field}
  with TheEditor do
  begin
    len:=SelLength;
    if (lpos > 0) and (len>1) and
       (Lines[lnumb][lpos] = '"') and
       (Lines[lnumb][lpos+len+1] = '"') then
    begin
      Dont_Update:=true;
      SelStart:=SelStart-1;
      SelLength := len+2;
      Dont_Update:=false;
    end;
  end;

end;

var macro_added:boolean=false;   // Whether we just addded a macro

procedure TEditorForm.TheEditorKeyPress(Sender: TObject; var Key: Char);
var lnumb,lpos,i,tl:integer;
    str:string;
begin
  {Handle the return key}
  if Key = Chr(13) then
  begin
    str := #13+#10;
    tl:=0;
    lnumb:= TheEditor.Perform(EM_LINEFROMCHAR,TheEditor.SelStart,0);
    while (tl<Length(TheEditor.Lines[lnumb])) and
            (TheEditor.Lines[lnumb][tl+1] = ' ') do
        tl:=tl+1;
    if AutoIndentCode then
      for i:=1 to tl do str:=str+' ';
    TheEditor.SelText := str;
    Key:=Chr(0);
    exit;
  end;

  {Handle the tab key}
  if Key = Chr(9) then
  begin
    lnumb:= TheEditor.Perform(EM_LINEFROMCHAR,TheEditor.SelStart,0);
    lpos := TheEditor.SelStart - TheEditor.Perform(EM_LINEINDEX,lnumb,0) + 1;
    if (lpos mod 2 = 0) then tl:=1 else tl:=2;
    if (lnumb > 0) then
    begin
      while (lpos+tl<=Length(TheEditor.Lines[lnumb-1])) and
            (TheEditor.Lines[lnumb-1][lpos+tl] = ' ') do
        tl:=tl+1;
    end;
    str:='';
    for i:=1 to tl do str:=str+' ';
    TheEditor.SelText := str;
    Key := Chr(0);
    exit;
  end;

  {Ignore <Ctr><Alt> combinations when a macro was added}
  if (macro_added) then
  begin
    Key := Chr(0);
    macro_added:=false;
    exit;
  end;

  {Rest is handles by the editor object itself}
end;

procedure TEditorForm.TheEditorKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var ctrldown,altdown,shiftdown:boolean;
    ch:char;
    i:integer;
    str,constr:string;
begin
  {Handle <Ctr><Alt> Combinations as macro's}
  ctrldown := (ssCtrl in Shift);
  altdown := (ssAlt in Shift);
  shiftdown := (ssShift in Shift);
  ch:=Char(MapVirtualKey(Key,2));
  if MacrosOn and ctrldown and altdown and
     (((ch>='A') and (ch<='Z')) or ((ch>='0') and (ch<='9'))) then
  begin
    str:='';
    if ctrldown then str:=str+'<Ctrl>';
    if altdown then str:=str+'<Alt>';
    if shiftdown then str:=str+'<Shift>';
    str:=str+ch;
    for i:=1 to macronumb do
    begin
      if Pos(str,Macros[i]) = 1 then
      begin
        constr:=Copy(Macros[i],Length(str)+2,1000);
        AddConstructString(constr);
        macro_added:=true;
        Key:=0;
        break;
      end;
    end;
  end;
end;


procedure TEditorForm.FormActivate(Sender: TObject);
begin
  TheEditorSelectionChange(Self);

  if TheErrors.Visible then
    MainForm.StatusBar1.Panels[1].Text := 'Errors'
  else
    MainForm.StatusBar1.Panels[1].Text := '';
  MainForm.SetToolBar(true);
end;

procedure TEditorForm.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  if TheEditor.Modified then
  begin
    BringToFront;
    case MessageDlg('File ' + Caption + ' has been modified. Save changes?',
            mtConfirmation, [mbYes, mbNo, mbCancel], 0) of
      id_Yes: MainForm.Save1Click(Self);
      id_Cancel: CanClose:=false;
    end;
  end;
end;

procedure TEditorForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action := caFree;
  MainForm.SetToolBar(false);
end;

procedure TEditorForm.FormShow(Sender: TObject);
begin
  PopupMenu:=ConstructForm.ConstructMenu;
  TheEditor.Color := BackgroundColor;
  TheEditor.Font.Name := FontName;
  TheEditor.Font.Size := FontSize;
end;

end.
